<script>
    export let size = '1em';
    export let color = 'currentColor';
</script>

<svg
    width={size}
    height={size}
    fill={color}
    viewBox="0 0 18 18"
    version="1.1"
    xmlns="http://www.w3.org/2000/svg"
    xmlns:xlink="http://www.w3.org/1999/xlink"
    xml:space="preserve"
    xmlns:serif="http://www.serif.com/"
    style="fill-rule:evenodd;clip-rule:evenodd;stroke-linejoin:round;stroke-miterlimit:2;"
>
    <path
        d="M14.657,7.343L9,13L3.343,7.343L4.757,5.929L9,10.172L13.243,5.929L14.657,7.343"
    />
</svg>
